"""TabRepo Rerun + manual changes.
# download from here:
# https://github.com/AutoML-Grandmasters/Fourth-AutoML-Grand-Prix/blob/main/tabrepo_2024_custom.py

We re-ran the work of the TabRepo paper, but use a portfolio of size 200 instead of the
100 size portfolio used by AutoGluon’s best_quality setting. We also included more model
families: Linear models and KNN.

# Removed because too slow predict for large datasets
LightGBM_r19
LightGBM_r96
LightGBM_r94_BAG_L1
LightGBM_r15_BAG_L1
LightGBM_r133_BAG_L1
LightGBM_r174
XGBoost_r31_BAG_L1
XGBoost_r33_BAG_L1

# Added manually:
LightGBM, XGBoost, and CatBoost with different max_bin values.
"""

from __future__ import annotations


zeroshot2024 = {
    "NN_TORCH": [
        {},
        {
            "activation": "elu",
            "dropout_prob": 0.10077639529843717,
            "hidden_size": 108,
            "learning_rate": 0.002735937344002146,
            "num_layers": 4,
            "use_batchnorm": True,
            "weight_decay": 1.356433327634438e-12,
            "ag_args": {"name_suffix": "_r79", "priority": -2},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.11897478034205347,
            "hidden_size": 213,
            "learning_rate": 0.0010474382260641949,
            "num_layers": 4,
            "use_batchnorm": False,
            "weight_decay": 5.594471067786272e-10,
            "ag_args": {"name_suffix": "_r22", "priority": -8},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.10276665747466765,
            "hidden_size": 165,
            "learning_rate": 0.0038022969797074676,
            "num_layers": 1,
            "use_batchnorm": False,
            "weight_decay": 8.916967887098654e-09,
            "ag_args": {"name_suffix": "_r164", "priority": -19},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.14717103221481653,
            "hidden_size": 222,
            "learning_rate": 0.002928569897768998,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 3.3157591552416304e-06,
            "ag_args": {"name_suffix": "_r115", "priority": -22},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.01030258381183309,
            "hidden_size": 111,
            "learning_rate": 0.01845979186513771,
            "num_layers": 5,
            "use_batchnorm": True,
            "weight_decay": 0.00020238017476912164,
            "ag_args": {"name_suffix": "_r158", "priority": -26},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.12415233039192458,
            "hidden_size": 131,
            "learning_rate": 0.016469659989857787,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 7.620839071729229e-06,
            "ag_args": {"name_suffix": "_r95", "priority": -30},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.3905837860053583,
            "hidden_size": 106,
            "learning_rate": 0.0018297905295930797,
            "num_layers": 1,
            "use_batchnorm": True,
            "weight_decay": 9.178069874232892e-08,
            "ag_args": {"name_suffix": "_r14", "priority": -41},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.3457125770744979,
            "hidden_size": 37,
            "learning_rate": 0.006435774191713849,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 2.4012185204155345e-08,
            "ag_args": {"name_suffix": "_r36", "priority": -48},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.10584781187588926,
            "hidden_size": 89,
            "learning_rate": 0.014791925972420345,
            "num_layers": 3,
            "use_batchnorm": False,
            "weight_decay": 2.0956656927351057e-11,
            "ag_args": {"name_suffix": "_r171", "priority": -50},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.3717900746799333,
            "hidden_size": 154,
            "learning_rate": 0.005908762713584694,
            "num_layers": 2,
            "use_batchnorm": True,
            "weight_decay": 5.681050909765253e-05,
            "ag_args": {"name_suffix": "_r200", "priority": -54},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.1703783780377607,
            "hidden_size": 212,
            "learning_rate": 0.0004107199833213839,
            "num_layers": 5,
            "use_batchnorm": True,
            "weight_decay": 1.105439140660822e-07,
            "ag_args": {"name_suffix": "_r143", "priority": -59},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.013288954106470907,
            "hidden_size": 81,
            "learning_rate": 0.005340914647396154,
            "num_layers": 4,
            "use_batchnorm": False,
            "weight_decay": 8.762168370775353e-05,
            "ag_args": {"name_suffix": "_r31", "priority": -64},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.15375268291707994,
            "hidden_size": 95,
            "learning_rate": 0.016181912728325788,
            "num_layers": 1,
            "use_batchnorm": False,
            "weight_decay": 1.6514168330341684e-08,
            "ag_args": {"name_suffix": "_r2", "priority": -71},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.059123470678908954,
            "hidden_size": 18,
            "learning_rate": 0.0002939890353572163,
            "num_layers": 5,
            "use_batchnorm": True,
            "weight_decay": 1.1744592227570546e-11,
            "ag_args": {"name_suffix": "_r122", "priority": -76},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.13685500504288658,
            "hidden_size": 229,
            "learning_rate": 0.0001934231631502052,
            "num_layers": 5,
            "use_batchnorm": True,
            "weight_decay": 1.0282176418614337e-05,
            "ag_args": {"name_suffix": "_r157", "priority": -83},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.2211285919550286,
            "hidden_size": 196,
            "learning_rate": 0.011307978270179143,
            "num_layers": 1,
            "use_batchnorm": True,
            "weight_decay": 1.8441764217351068e-06,
            "ag_args": {"name_suffix": "_r19", "priority": -85},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.36669080773207274,
            "hidden_size": 95,
            "learning_rate": 0.015280159186761077,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 1.3082489374636015e-08,
            "ag_args": {"name_suffix": "_r87", "priority": -94},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.23713784729000734,
            "hidden_size": 200,
            "learning_rate": 0.00311256170909018,
            "num_layers": 4,
            "use_batchnorm": True,
            "weight_decay": 4.573016756474468e-08,
            "ag_args": {"name_suffix": "_r1", "priority": -97},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.06134755114373829,
            "hidden_size": 144,
            "learning_rate": 0.005834535148903801,
            "num_layers": 5,
            "use_batchnorm": True,
            "weight_decay": 2.0826540090463355e-09,
            "ag_args": {"name_suffix": "_r135", "priority": -101},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.09976801642258049,
            "hidden_size": 135,
            "learning_rate": 0.001631450730978947,
            "num_layers": 5,
            "use_batchnorm": False,
            "weight_decay": 3.867683394425807e-05,
            "ag_args": {"name_suffix": "_r86", "priority": -103},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.3027114570947557,
            "hidden_size": 196,
            "learning_rate": 0.006482759295309238,
            "num_layers": 1,
            "use_batchnorm": False,
            "weight_decay": 1.2806509958776e-12,
            "ag_args": {"name_suffix": "_r71", "priority": -109},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.01759017280963624,
            "hidden_size": 250,
            "learning_rate": 0.00027701426675638685,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 2.941370491259882e-10,
            "ag_args": {"name_suffix": "_r190", "priority": -111},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.14075719847721704,
            "hidden_size": 107,
            "learning_rate": 0.02102016621542758,
            "num_layers": 5,
            "use_batchnorm": True,
            "weight_decay": 8.583524549723169e-10,
            "ag_args": {"name_suffix": "_r117", "priority": -112},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.34251047184911276,
            "hidden_size": 199,
            "learning_rate": 0.00013693277454657866,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 0.005425887964567897,
            "ag_args": {"name_suffix": "_r134", "priority": -120},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.2232260403696044,
            "hidden_size": 10,
            "learning_rate": 0.00016660531025826497,
            "num_layers": 2,
            "use_batchnorm": True,
            "weight_decay": 1.851165282762072e-12,
            "ag_args": {"name_suffix": "_r100", "priority": -129},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.383666641214089,
            "hidden_size": 14,
            "learning_rate": 0.0017662443351764005,
            "num_layers": 2,
            "use_batchnorm": False,
            "weight_decay": 1.180743534035588e-11,
            "ag_args": {"name_suffix": "_r72", "priority": -135},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.006531401073483156,
            "hidden_size": 192,
            "learning_rate": 0.012418052210914356,
            "num_layers": 1,
            "use_batchnorm": True,
            "weight_decay": 3.0406866089493607e-05,
            "ag_args": {"name_suffix": "_r76", "priority": -138},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.35413506384383425,
            "hidden_size": 128,
            "learning_rate": 0.02291819787571232,
            "num_layers": 1,
            "use_batchnorm": True,
            "weight_decay": 2.347385665763067e-10,
            "ag_args": {"name_suffix": "_r43", "priority": -145},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.13744220125923948,
            "hidden_size": 245,
            "learning_rate": 0.01760548103080169,
            "num_layers": 4,
            "use_batchnorm": True,
            "weight_decay": 4.619518091833427e-09,
            "ag_args": {"name_suffix": "_r57", "priority": -147},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.33567564890346097,
            "hidden_size": 245,
            "learning_rate": 0.006746560197328548,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 1.6470047305392933e-10,
            "ag_args": {"name_suffix": "_r89", "priority": -148},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.3834963692226372,
            "hidden_size": 30,
            "learning_rate": 0.0006387876089505596,
            "num_layers": 4,
            "use_batchnorm": True,
            "weight_decay": 0.020005485780993044,
            "ag_args": {"name_suffix": "_r120", "priority": -153},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.12166942295569863,
            "hidden_size": 151,
            "learning_rate": 0.0018866871631794007,
            "num_layers": 4,
            "use_batchnorm": True,
            "weight_decay": 9.190843763153802e-05,
            "ag_args": {"name_suffix": "_r185", "priority": -157},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.17934767155919895,
            "hidden_size": 26,
            "learning_rate": 0.0002670236000453949,
            "num_layers": 5,
            "use_batchnorm": False,
            "weight_decay": 9.550309584226972e-08,
            "ag_args": {"name_suffix": "_r169", "priority": -158},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.28421136108893824,
            "hidden_size": 233,
            "learning_rate": 0.004756576344566884,
            "num_layers": 2,
            "use_batchnorm": True,
            "weight_decay": 0.004694424874123613,
            "ag_args": {"name_suffix": "_r98", "priority": -165},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.19838811814938784,
            "hidden_size": 91,
            "learning_rate": 0.0018766153235863243,
            "num_layers": 5,
            "use_batchnorm": True,
            "weight_decay": 1.9551641972196325e-11,
            "ag_args": {"name_suffix": "_r139", "priority": -167},
        },
        {
            "activation": "elu",
            "dropout_prob": 0.08014572877647762,
            "hidden_size": 238,
            "learning_rate": 0.003193230959659184,
            "num_layers": 4,
            "use_batchnorm": False,
            "weight_decay": 0.006268749322131784,
            "ag_args": {"name_suffix": "_r114", "priority": -168},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.22890076231634937,
            "hidden_size": 188,
            "learning_rate": 0.016161752294108052,
            "num_layers": 1,
            "use_batchnorm": False,
            "weight_decay": 0.0581139255429484,
            "ag_args": {"name_suffix": "_r21", "priority": -181},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.31096302473950127,
            "hidden_size": 115,
            "learning_rate": 0.00027659068642753017,
            "num_layers": 4,
            "use_batchnorm": True,
            "weight_decay": 1.2781148440039829e-12,
            "ag_args": {"name_suffix": "_r64", "priority": -183},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.05930745685905112,
            "hidden_size": 14,
            "learning_rate": 0.00011882187473200266,
            "num_layers": 4,
            "use_batchnorm": False,
            "weight_decay": 1.2573893223363365e-08,
            "ag_args": {"name_suffix": "_r99", "priority": -191},
        },
        {
            "activation": "relu",
            "dropout_prob": 0.22187512346456761,
            "hidden_size": 97,
            "learning_rate": 0.01090384075529055,
            "num_layers": 3,
            "use_batchnorm": True,
            "weight_decay": 9.24700655189339e-09,
            "ag_args": {"name_suffix": "_r83", "priority": -192},
        },
    ],
    "GBM": [
        {},
        {  # Added manually.
            "max_bin": 4095,
            "ag_args": {"priority": -1, "name_suffix": "Bin4095"},
        },
        {"extra_trees": True, "ag_args": {"name_suffix": "XT"}},
        {  # Old GBMLarge
            "learning_rate": 0.03,
            "num_leaves": 128,
            "feature_fraction": 0.9,
            "min_data_in_leaf": 3,
            "ag_args": {
                "name_suffix": "Large",
                "priority": 0,
                "hyperparameter_tune_kwargs": None,
            },
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.7023601671276614,
            "learning_rate": 0.012144796373999013,
            "min_data_in_leaf": 14,
            "num_leaves": 53,
            "ag_args": {"name_suffix": "_r131", "priority": -3},
        },
        # { # TOo slow for large datasets
        #     "extra_trees": True,
        #     "feature_fraction": 0.5636931414546802,
        #     "learning_rate": 0.01518660230385841,
        #     "min_data_in_leaf": 48,
        #     "num_leaves": 16,
        #     "ag_args": {"name_suffix": "_r96", "priority": -7},
        # },
        # { # Too slow
        #     "extra_trees": True,
        #     "feature_fraction": 0.7291114577678862,
        #     "learning_rate": 0.02371597110187949,
        #     "min_data_in_leaf": 5,
        #     "num_leaves": 90,
        #     "ag_args": {"name_suffix": "_r133", "priority": -13},
        # },
        # { # Too slow
        #     "extra_trees": False,
        #     "feature_fraction": 0.7421180622507277,
        #     "learning_rate": 0.018603888565740096,
        #     "min_data_in_leaf": 6,
        #     "num_leaves": 22,
        #     "ag_args": {"name_suffix": "_r15", "priority": -15},
        # },
        {
            "extra_trees": True,
            "feature_fraction": 0.8999894845710796,
            "learning_rate": 0.051087336729504676,
            "min_data_in_leaf": 18,
            "num_leaves": 167,
            "ag_args": {"name_suffix": "_r191", "priority": -24},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.7325454610506641,
            "learning_rate": 0.009447054356012436,
            "min_data_in_leaf": 4,
            "num_leaves": 85,
            "ag_args": {"name_suffix": "_r54", "priority": -27},
        },
        {
            "extra_trees": True,
            "feature_fraction": 0.8682559906624081,
            "learning_rate": 0.09561511371136407,
            "min_data_in_leaf": 9,
            "num_leaves": 121,
            "ag_args": {"name_suffix": "_r81", "priority": -28},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.8254432681390782,
            "learning_rate": 0.031251656439648626,
            "min_data_in_leaf": 50,
            "num_leaves": 210,
            "ag_args": {"name_suffix": "_r135", "priority": -32},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.9668244885378855,
            "learning_rate": 0.07254551525590439,
            "min_data_in_leaf": 14,
            "num_leaves": 31,
            "ag_args": {"name_suffix": "_r145", "priority": -40},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.45835595623790437,
            "learning_rate": 0.09533195017847339,
            "min_data_in_leaf": 7,
            "num_leaves": 231,
            "ag_args": {"name_suffix": "_r41", "priority": -44},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.6245777099925497,
            "learning_rate": 0.04711573688184715,
            "min_data_in_leaf": 56,
            "num_leaves": 89,
            "ag_args": {"name_suffix": "_r130", "priority": -52},
        },
        # { # Too slow for large datasets
        #     "extra_trees": True,
        #     "feature_fraction": 0.5179494170080321,
        #     "learning_rate": 0.015090113680567405,
        #     "min_data_in_leaf": 7,
        #     "num_leaves": 110,
        #     "ag_args": {"name_suffix": "_r19", "priority": -62},
        # },
        # { # Too slow for large datasets
        #     "extra_trees": True,
        #     "feature_fraction": 0.4341088458599442,
        #     "learning_rate": 0.04034449862560467,
        #     "min_data_in_leaf": 33,
        #     "num_leaves": 16,
        #     "ag_args": {"name_suffix": "_r94", "priority": -65},
        # },
        {
            "extra_trees": True,
            "feature_fraction": 0.9666234339903601,
            "learning_rate": 0.04582977995120822,
            "min_data_in_leaf": 4,
            "num_leaves": 127,
            "ag_args": {"name_suffix": "_r55", "priority": -68},
        },
        # { # Too slow
        #     "extra_trees": False,
        #     "feature_fraction": 0.9995667963533027,
        #     "learning_rate": 0.01434806540259691,
        #     "min_data_in_leaf": 37,
        #     "num_leaves": 183,
        #     "ag_args": {"name_suffix": "_r174", "priority": -88},
        # },
        {
            "extra_trees": False,
            "feature_fraction": 0.7016257244614168,
            "learning_rate": 0.007922167829715967,
            "min_data_in_leaf": 7,
            "num_leaves": 132,
            "ag_args": {"name_suffix": "_r149", "priority": -91},
        },
        {
            "extra_trees": True,
            "feature_fraction": 0.9046840778713597,
            "learning_rate": 0.07515257316211908,
            "min_data_in_leaf": 42,
            "num_leaves": 18,
            "ag_args": {"name_suffix": "_r43", "priority": -100},
        },
        {
            "extra_trees": True,
            "feature_fraction": 0.4601361323873807,
            "learning_rate": 0.07856777698860955,
            "min_data_in_leaf": 12,
            "num_leaves": 198,
            "ag_args": {"name_suffix": "_r42", "priority": -105},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.7532437659821729,
            "learning_rate": 0.08944644189688526,
            "min_data_in_leaf": 39,
            "num_leaves": 53,
            "ag_args": {"name_suffix": "_r153", "priority": -118},
        },
        {
            "extra_trees": True,
            "feature_fraction": 0.43613528297756193,
            "learning_rate": 0.03685135839677242,
            "min_data_in_leaf": 57,
            "num_leaves": 27,
            "ag_args": {"name_suffix": "_r13", "priority": -121},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.7579932437770318,
            "learning_rate": 0.052301563688720604,
            "min_data_in_leaf": 37,
            "num_leaves": 136,
            "ag_args": {"name_suffix": "_r51", "priority": -131},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.41239059967943725,
            "learning_rate": 0.04848901712678711,
            "min_data_in_leaf": 5,
            "num_leaves": 67,
            "ag_args": {"name_suffix": "_r61", "priority": -132},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.40585986135777,
            "learning_rate": 0.012590980616372347,
            "min_data_in_leaf": 32,
            "num_leaves": 22,
            "ag_args": {"name_suffix": "_r106", "priority": -139},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.9744705133953723,
            "learning_rate": 0.020546267996855768,
            "min_data_in_leaf": 60,
            "num_leaves": 99,
            "ag_args": {"name_suffix": "_r66", "priority": -163},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.6937293621346563,
            "learning_rate": 0.013803836586316339,
            "min_data_in_leaf": 38,
            "num_leaves": 16,
            "ag_args": {"name_suffix": "_r49", "priority": -164},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.6090855934200983,
            "learning_rate": 0.04590490414627263,
            "min_data_in_leaf": 56,
            "num_leaves": 144,
            "ag_args": {"name_suffix": "_r144", "priority": -171},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.5730390983988963,
            "learning_rate": 0.010305352949119608,
            "min_data_in_leaf": 10,
            "num_leaves": 215,
            "ag_args": {"name_suffix": "_r121", "priority": -172},
        },
        {
            "extra_trees": False,
            "feature_fraction": 0.45118655387122203,
            "learning_rate": 0.009705399613761859,
            "min_data_in_leaf": 9,
            "num_leaves": 45,
            "ag_args": {"name_suffix": "_r198", "priority": -173},
        },
    ],
    "CAT": [
        {},
        {
            # Added manually.
            "max_bin": 4095,
            "ag_args": {"priority": -3, "name_suffix": "Bin4095"},
        },
        {
            "depth": 6,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 2.1542798306067823,
            "learning_rate": 0.06864209415792857,
            "max_ctr_complexity": 4,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r177", "priority": -1},
        },
        {
            "depth": 8,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 2.7997999596449104,
            "learning_rate": 0.031375015734637225,
            "max_ctr_complexity": 2,
            "one_hot_max_size": 3,
            "ag_args": {"name_suffix": "_r9", "priority": -5},
        },
        {
            "depth": 4,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.559174625782161,
            "learning_rate": 0.04939557741379516,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 3,
            "ag_args": {"name_suffix": "_r137", "priority": -12},
        },
        {
            "depth": 8,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 3.3274013177541373,
            "learning_rate": 0.017301189655111057,
            "max_ctr_complexity": 5,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r13", "priority": -18},
        },
        {
            "depth": 4,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 1.9842527399638579,
            "learning_rate": 0.03675129858005787,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r95", "priority": -20},
        },
        {
            "depth": 8,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 2.9106604692853995,
            "learning_rate": 0.05696733223175933,
            "max_ctr_complexity": 1,
            "one_hot_max_size": 3,
            "ag_args": {"name_suffix": "_r3", "priority": -29},
        },
        {
            "depth": 6,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 1.3584121369544215,
            "learning_rate": 0.03743901034980473,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r70", "priority": -34},
        },
        {
            "depth": 4,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 3.353268454214423,
            "learning_rate": 0.06028218319511302,
            "max_ctr_complexity": 1,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r49", "priority": -38},
        },
        {
            "depth": 7,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 3.609235810966101,
            "learning_rate": 0.05862063297323188,
            "max_ctr_complexity": 2,
            "one_hot_max_size": 5,
            "ag_args": {"name_suffix": "_r135", "priority": -42},
        },
        {
            "depth": 8,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 1.3976655774456335,
            "learning_rate": 0.0119117922642152,
            "max_ctr_complexity": 2,
            "one_hot_max_size": 3,
            "ag_args": {"name_suffix": "_r118", "priority": -49},
        },
        {
            "depth": 8,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.487142939021972,
            "learning_rate": 0.022543679111584062,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r124", "priority": -53},
        },
        {
            "depth": 5,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 1.0457098345001241,
            "learning_rate": 0.050294288910022224,
            "max_ctr_complexity": 5,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r69", "priority": -56},
        },
        {
            "depth": 7,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 3.7631391046261182,
            "learning_rate": 0.09628094704738965,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 3,
            "ag_args": {"name_suffix": "_r159", "priority": -60},
        },
        {
            "depth": 4,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 2.7018061518087038,
            "learning_rate": 0.07092851311746352,
            "max_ctr_complexity": 1,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r50", "priority": -78},
        },
        {
            "depth": 6,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 3.637071465711953,
            "learning_rate": 0.04387418552563314,
            "max_ctr_complexity": 4,
            "one_hot_max_size": 5,
            "ag_args": {"name_suffix": "_r198", "priority": -80},
        },
        {
            "depth": 7,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.835797074498082,
            "learning_rate": 0.03534026385152556,
            "max_ctr_complexity": 5,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r12", "priority": -87},
        },
        {
            "depth": 7,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 4.693224271556653,
            "learning_rate": 0.06007466728599504,
            "max_ctr_complexity": 5,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r125", "priority": -92},
        },
        {
            "depth": 4,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.901696537625293,
            "learning_rate": 0.05205851115876207,
            "max_ctr_complexity": 2,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r171", "priority": -95},
        },
        {
            "depth": 5,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 3.7454481983750014,
            "learning_rate": 0.09328642499990342,
            "max_ctr_complexity": 1,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r163", "priority": -98},
        },
        {
            "depth": 6,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 1.3843936315758523,
            "learning_rate": 0.09319080384124982,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 5,
            "ag_args": {"name_suffix": "_r16", "priority": -104},
        },
        {
            "depth": 8,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 1.640921865280573,
            "learning_rate": 0.036232951900213306,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 5,
            "ag_args": {"name_suffix": "_r128", "priority": -107},
        },
        {
            "depth": 4,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 2.894432181094842,
            "learning_rate": 0.055078095725390575,
            "max_ctr_complexity": 4,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r5", "priority": -110},
        },
        {
            "depth": 6,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 3.218751234645676,
            "learning_rate": 0.011814970512055711,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 3,
            "ag_args": {"name_suffix": "_r96", "priority": -122},
        },
        {
            "depth": 7,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 1.6761016245166451,
            "learning_rate": 0.06566144806528762,
            "max_ctr_complexity": 2,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r143", "priority": -133},
        },
        {
            "depth": 8,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 3.5217917474671645,
            "learning_rate": 0.0686190772732043,
            "max_ctr_complexity": 2,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r62", "priority": -142},
        },
        {
            "depth": 7,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 4.43335055453705,
            "learning_rate": 0.055406199833457785,
            "max_ctr_complexity": 5,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r180", "priority": -149},
        },
        {
            "depth": 8,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.829803422635814,
            "learning_rate": 0.020546267996855768,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r52", "priority": -154},
        },
        {
            "depth": 7,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.522712492188319,
            "learning_rate": 0.08481607830570326,
            "max_ctr_complexity": 3,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r167", "priority": -156},
        },
        {
            "depth": 5,
            "grow_policy": "Depthwise",
            "l2_leaf_reg": 1.2612168286071208,
            "learning_rate": 0.05223739392014652,
            "max_ctr_complexity": 4,
            "one_hot_max_size": 5,
            "ag_args": {"name_suffix": "_r94", "priority": -162},
        },
        {
            "depth": 4,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 1.9515712854980345,
            "learning_rate": 0.08211271991437913,
            "max_ctr_complexity": 5,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r41", "priority": -170},
        },
        {
            "depth": 5,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.299190822458588,
            "learning_rate": 0.037054953070982596,
            "max_ctr_complexity": 4,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r183", "priority": -175},
        },
        {
            "depth": 6,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 4.3146276582229515,
            "learning_rate": 0.079869267564513,
            "max_ctr_complexity": 1,
            "one_hot_max_size": 10,
            "ag_args": {"name_suffix": "_r97", "priority": -176},
        },
        {
            "depth": 7,
            "grow_policy": "SymmetricTree",
            "l2_leaf_reg": 1.2565899853951374,
            "learning_rate": 0.039801276672212574,
            "max_ctr_complexity": 4,
            "one_hot_max_size": 2,
            "ag_args": {"name_suffix": "_r19", "priority": -186},
        },
    ],
    "XGB": [
        {},
        {
            # Added manually.
            "max_bin": 4095,
            "ag_args": {"priority": -2, "name_suffix": "Bin4095"},
        },
        {
            # Added manually.
            "max_bin": 16383,
            "ag_args": {"priority": -4, "name_suffix": "Bin4095"},
        },
        # {  # Too slow
        #     "colsample_bytree": 0.6917311125174739,
        #     "enable_categorical": False,
        #     "learning_rate": 0.018063876087523967,
        #     "max_depth": 10,
        #     "min_child_weight": 0.6028633586934382,
        #     "ag_args": {"name_suffix": "_r33", "priority": -9},
        # },
        {
            "colsample_bytree": 0.6326947454697227,
            "enable_categorical": False,
            "learning_rate": 0.07792091886639502,
            "max_depth": 6,
            "min_child_weight": 1.0759464955561793,
            "ag_args": {"name_suffix": "_r22", "priority": -25},
        },
        {
            "colsample_bytree": 0.8387059147359721,
            "enable_categorical": True,
            "learning_rate": 0.010427977953050884,
            "max_depth": 7,
            "min_child_weight": 1.2521345209879338,
            "ag_args": {"name_suffix": "_r78", "priority": -35},
        },
        {
            "colsample_bytree": 0.922117480776512,
            "enable_categorical": True,
            "learning_rate": 0.09278141584600721,
            "max_depth": 10,
            "min_child_weight": 1.3428548878354571,
            "ag_args": {"name_suffix": "_r193", "priority": -47},
        },
        {
            "colsample_bytree": 0.7452294043087835,
            "enable_categorical": False,
            "learning_rate": 0.038404229910104046,
            "max_depth": 7,
            "min_child_weight": 0.5564183327139662,
            "ag_args": {"name_suffix": "_r49", "priority": -51},
        },
        # { # Too slow
        #     "colsample_bytree": 0.7506621909633511,
        #     "enable_categorical": False,
        #     "learning_rate": 0.009974712407899168,
        #     "max_depth": 4,
        #     "min_child_weight": 0.9238550485581797,
        #     "ag_args": {"name_suffix": "_r31", "priority": -55},
        # },
        {
            "colsample_bytree": 0.5078030322234142,
            "enable_categorical": True,
            "learning_rate": 0.03502844787947471,
            "max_depth": 9,
            "min_child_weight": 0.7519409882460929,
            "ag_args": {"name_suffix": "_r102", "priority": -69},
        },
        {
            "colsample_bytree": 0.975937238416368,
            "enable_categorical": False,
            "learning_rate": 0.06634196266155237,
            "max_depth": 5,
            "min_child_weight": 1.4088437184127383,
            "ag_args": {"name_suffix": "_r95", "priority": -90},
        },
        {
            "colsample_bytree": 0.8551684144871067,
            "enable_categorical": False,
            "learning_rate": 0.01984858936348865,
            "max_depth": 10,
            "min_child_weight": 1.0305372145627818,
            "ag_args": {"name_suffix": "_r130", "priority": -116},
        },
        {
            "colsample_bytree": 0.7317254886922415,
            "enable_categorical": True,
            "learning_rate": 0.014069637236426217,
            "max_depth": 4,
            "min_child_weight": 1.108252865854013,
            "ag_args": {"name_suffix": "_r44", "priority": -117},
        },
        {
            "colsample_bytree": 0.6363281472900566,
            "enable_categorical": False,
            "learning_rate": 0.05358312362730344,
            "max_depth": 10,
            "min_child_weight": 1.0680445610939322,
            "ag_args": {"name_suffix": "_r3", "priority": -119},
        },
        {
            "colsample_bytree": 0.5019301757551305,
            "enable_categorical": False,
            "learning_rate": 0.028332242590352767,
            "max_depth": 5,
            "min_child_weight": 0.5813695988533053,
            "ag_args": {"name_suffix": "_r184", "priority": -124},
        },
        {
            "colsample_bytree": 0.6167101277340481,
            "enable_categorical": True,
            "learning_rate": 0.01585533568837928,
            "max_depth": 6,
            "min_child_weight": 1.230709099127476,
            "ag_args": {"name_suffix": "_r113", "priority": -127},
        },
        {
            "colsample_bytree": 0.5619099914247208,
            "enable_categorical": False,
            "learning_rate": 0.06672278511715538,
            "max_depth": 9,
            "min_child_weight": 1.0691007386145932,
            "ag_args": {"name_suffix": "_r39", "priority": -146},
        },
        {
            "colsample_bytree": 0.8576039170061793,
            "enable_categorical": False,
            "learning_rate": 0.012446976122406065,
            "max_depth": 8,
            "min_child_weight": 0.5030995410843868,
            "ag_args": {"name_suffix": "_r147", "priority": -184},
        },
        {
            "colsample_bytree": 0.9679160401083943,
            "enable_categorical": True,
            "learning_rate": 0.07231523643035528,
            "max_depth": 9,
            "min_child_weight": 0.7115338977263024,
            "ag_args": {"name_suffix": "_r159", "priority": -185},
        },
    ],
    "FASTAI": [
        {},
        {
            "bs": 256,
            "emb_drop": 0.5411770367537934,
            "epochs": 43,
            "layers": [800, 400],
            "lr": 0.01519848858318159,
            "ps": 0.23782946566604385,
            "ag_args": {"name_suffix": "_r191", "priority": -4},
        },
        {
            "bs": 128,
            "emb_drop": 0.3939481870086508,
            "epochs": 50,
            "layers": [400, 200, 100],
            "lr": 0.05474993731798857,
            "ps": 0.312476090838242,
            "ag_args": {"name_suffix": "_r120", "priority": -10},
        },
        {
            "bs": 128,
            "emb_drop": 0.44339037504795686,
            "epochs": 31,
            "layers": [400, 200, 100],
            "lr": 0.008615195908919904,
            "ps": 0.19220253419114286,
            "ag_args": {"name_suffix": "_r145", "priority": -14},
        },
        {
            "bs": 2048,
            "emb_drop": 0.4149912327575128,
            "epochs": 20,
            "layers": [400, 200],
            "lr": 0.04711168148637163,
            "ps": 0.5930762171488877,
            "ag_args": {"name_suffix": "_r1", "priority": -17},
        },
        {
            "bs": 128,
            "emb_drop": 0.026897798530914306,
            "epochs": 31,
            "layers": [800, 400],
            "lr": 0.08045277634470181,
            "ps": 0.4569532219038436,
            "ag_args": {"name_suffix": "_r11", "priority": -23},
        },
        {
            "bs": 1024,
            "emb_drop": 0.5074958658302495,
            "epochs": 42,
            "layers": [200, 100, 50],
            "lr": 0.026342427824862867,
            "ps": 0.34814978753283593,
            "ag_args": {"name_suffix": "_r187", "priority": -33},
        },
        {
            "bs": 256,
            "emb_drop": 0.0052278224214431955,
            "epochs": 39,
            "layers": [200, 100, 50],
            "lr": 0.019675935745941037,
            "ps": 0.49360327844735585,
            "ag_args": {"name_suffix": "_r165", "priority": -37},
        },
        {
            "bs": 128,
            "emb_drop": 0.08669109226243704,
            "epochs": 45,
            "layers": [800, 400],
            "lr": 0.0041554361714983635,
            "ps": 0.2669780074016213,
            "ag_args": {"name_suffix": "_r138", "priority": -39},
        },
        {
            "bs": 512,
            "emb_drop": 0.5325204955732282,
            "epochs": 36,
            "layers": [200],
            "lr": 0.021387722847117794,
            "ps": 0.41116271561673834,
            "ag_args": {"name_suffix": "_r179", "priority": -43},
        },
        {
            "bs": 512,
            "emb_drop": 0.304332450990704,
            "epochs": 37,
            "layers": [400, 200],
            "lr": 0.022114884908502042,
            "ps": 0.3661044725262556,
            "ag_args": {"name_suffix": "_r123", "priority": -45},
        },
        {
            "bs": 2048,
            "emb_drop": 0.006251885504130949,
            "epochs": 47,
            "layers": [800, 400],
            "lr": 0.01329622020483052,
            "ps": 0.2677080696008348,
            "ag_args": {"name_suffix": "_r134", "priority": -58},
        },
        {
            "bs": 128,
            "emb_drop": 0.09607594536689695,
            "epochs": 36,
            "layers": [400, 200],
            "lr": 0.05050764650138042,
            "ps": 0.12558704891144218,
            "ag_args": {"name_suffix": "_r131", "priority": -63},
        },
        {
            "bs": 512,
            "emb_drop": 0.05604276533830355,
            "epochs": 32,
            "layers": [400],
            "lr": 0.027320709383189166,
            "ps": 0.022591301744255762,
            "ag_args": {"name_suffix": "_r172", "priority": -70},
        },
        {
            "bs": 128,
            "emb_drop": 0.6656668277387758,
            "epochs": 32,
            "layers": [400, 200, 100],
            "lr": 0.019326244622675428,
            "ps": 0.04084945128641206,
            "ag_args": {"name_suffix": "_r95", "priority": -73},
        },
        {
            "bs": 256,
            "emb_drop": 0.6539497985473556,
            "epochs": 27,
            "layers": [800, 400],
            "lr": 0.008540379133171428,
            "ps": 0.4129369834481997,
            "ag_args": {"name_suffix": "_r39", "priority": -77},
        },
        {
            "bs": 1024,
            "emb_drop": 0.4329361816589235,
            "epochs": 50,
            "layers": [400],
            "lr": 0.09501311551121323,
            "ps": 0.2863378667611431,
            "ag_args": {"name_suffix": "_r88", "priority": -81},
        },
        {
            "bs": 512,
            "emb_drop": 0.48799913177984827,
            "epochs": 37,
            "layers": [200, 100, 50],
            "lr": 0.09585563123235417,
            "ps": 0.17459402873951585,
            "ag_args": {"name_suffix": "_r94", "priority": -84},
        },
        {
            "bs": 1024,
            "emb_drop": 0.03949283289977636,
            "epochs": 37,
            "layers": [400, 200, 100],
            "lr": 0.05323385634320209,
            "ps": 0.6834830706151297,
            "ag_args": {"name_suffix": "_r42", "priority": -93},
        },
        {
            "bs": 128,
            "emb_drop": 0.013991365786131115,
            "epochs": 30,
            "layers": [200, 100],
            "lr": 0.08974875871045038,
            "ps": 0.25161112477852504,
            "ag_args": {"name_suffix": "_r16", "priority": -96},
        },
        {
            "bs": 256,
            "emb_drop": 0.6036229827851316,
            "epochs": 26,
            "layers": [400, 200],
            "lr": 0.0019169423883858754,
            "ps": 0.3122948588614413,
            "ag_args": {"name_suffix": "_r106", "priority": -102},
        },
        {
            "bs": 512,
            "emb_drop": 0.3132611095327173,
            "epochs": 22,
            "layers": [800, 400],
            "lr": 0.0037111611381388542,
            "ps": 0.07002243450258397,
            "ag_args": {"name_suffix": "_r101", "priority": -108},
        },
        {
            "bs": 1024,
            "emb_drop": 0.31956392388385874,
            "epochs": 25,
            "layers": [200, 100],
            "lr": 0.08552736732040143,
            "ps": 0.0934076022219228,
            "ag_args": {"name_suffix": "_r127", "priority": -113},
        },
        {
            "bs": 128,
            "emb_drop": 0.38627138139328115,
            "epochs": 42,
            "layers": [800, 400],
            "lr": 0.08483598191521528,
            "ps": 0.22469806896144823,
            "ag_args": {"name_suffix": "_r29", "priority": -115},
        },
        {
            "bs": 2048,
            "emb_drop": 0.5482641068196692,
            "epochs": 47,
            "layers": [400, 200, 100],
            "lr": 0.0017967245032803941,
            "ps": 0.46375320007287313,
            "ag_args": {"name_suffix": "_r90", "priority": -123},
        },
        {
            "bs": 256,
            "emb_drop": 0.3358277091379548,
            "epochs": 39,
            "layers": [400, 200],
            "lr": 0.08407418785434553,
            "ps": 0.06997096091499505,
            "ag_args": {"name_suffix": "_r136", "priority": -125},
        },
        {
            "bs": 128,
            "emb_drop": 0.1714226355940696,
            "epochs": 45,
            "layers": [200],
            "lr": 0.0012051775395048954,
            "ps": 0.6459296271577557,
            "ag_args": {"name_suffix": "_r159", "priority": -130},
        },
        {
            "bs": 2048,
            "emb_drop": 0.000382175427896958,
            "epochs": 46,
            "layers": [400, 200, 100],
            "lr": 0.002028652260418399,
            "ps": 0.24384972715881284,
            "ag_args": {"name_suffix": "_r169", "priority": -134},
        },
        {
            "bs": 128,
            "emb_drop": 0.4599138419358,
            "epochs": 47,
            "layers": [200, 100],
            "lr": 0.03888383281136287,
            "ps": 0.28193673177122863,
            "ag_args": {"name_suffix": "_r128", "priority": -143},
        },
        {
            "bs": 1024,
            "emb_drop": 0.22771721361129746,
            "epochs": 38,
            "layers": [400],
            "lr": 0.0005383511954451698,
            "ps": 0.3734259772256502,
            "ag_args": {"name_suffix": "_r65", "priority": -144},
        },
        {
            "bs": 128,
            "emb_drop": 0.1978897556618756,
            "epochs": 49,
            "layers": [400, 200, 100],
            "lr": 0.021551443035084644,
            "ps": 0.005518872455908264,
            "ag_args": {"name_suffix": "_r109", "priority": -152},
        },
        {
            "bs": 512,
            "emb_drop": 0.3794906175059568,
            "epochs": 49,
            "layers": [400, 200],
            "lr": 0.01841936985537725,
            "ps": 0.2009834831789172,
            "ag_args": {"name_suffix": "_r118", "priority": -160},
        },
        {
            "bs": 512,
            "emb_drop": 0.24892891649496893,
            "epochs": 48,
            "layers": [800, 400],
            "lr": 0.02884073803446048,
            "ps": 0.5240645338953831,
            "ag_args": {"name_suffix": "_r53", "priority": -161},
        },
        {
            "bs": 256,
            "emb_drop": 0.2684235575244635,
            "epochs": 29,
            "layers": [200, 100],
            "lr": 0.052518805993807795,
            "ps": 0.07200435108540675,
            "ag_args": {"name_suffix": "_r28", "priority": -166},
        },
        {
            "bs": 1024,
            "emb_drop": 0.0463046234818683,
            "epochs": 36,
            "layers": [800, 400],
            "lr": 0.04669680619450405,
            "ps": 0.11348418249618325,
            "ag_args": {"name_suffix": "_r146", "priority": -174},
        },
        {
            "bs": 256,
            "emb_drop": 0.6272268712563539,
            "epochs": 44,
            "layers": [800, 400],
            "lr": 0.05629418126052196,
            "ps": 0.4760388986243886,
            "ag_args": {"name_suffix": "_r23", "priority": -182},
        },
        {
            "bs": 1024,
            "emb_drop": 0.4530218981312888,
            "epochs": 42,
            "layers": [200],
            "lr": 0.002032928260082698,
            "ps": 0.2784745269310403,
            "ag_args": {"name_suffix": "_r20", "priority": -188},
        },
        {
            "bs": 512,
            "emb_drop": 0.4875242012567504,
            "epochs": 36,
            "layers": [400, 200],
            "lr": 0.0006167746422948765,
            "ps": 0.041961008774701276,
            "ag_args": {"name_suffix": "_r122", "priority": -189},
        },
        {
            "bs": 2048,
            "emb_drop": 0.6343202884164582,
            "epochs": 21,
            "layers": [400, 200],
            "lr": 0.08479209380262258,
            "ps": 0.48362560779595565,
            "ag_args": {"name_suffix": "_r111", "priority": -190},
        },
        {
            "bs": 128,
            "emb_drop": 0.5990623396748277,
            "epochs": 33,
            "layers": [200, 100, 50],
            "lr": 0.023916641022656154,
            "ps": 0.12014077408300836,
            "ag_args": {"name_suffix": "_r35", "priority": -193},
        },
    ],
    "RF": [
        {
            "criterion": "gini",
            "ag_args": {
                "name_suffix": "Gini",
                "problem_types": ["binary", "multiclass"],
            },
        },
        {
            "criterion": "entropy",
            "ag_args": {
                "name_suffix": "Entr",
                "problem_types": ["binary", "multiclass"],
            },
        },
        {
            "criterion": "squared_error",
            "ag_args": {
                "name_suffix": "MSE",
                "problem_types": ["regression", "quantile"],
            },
        },
        {
            "max_features": 0.75,
            "max_leaf_nodes": 18392,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r42", "priority": -11},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 25177,
            "min_samples_leaf": 5,
            "ag_args": {"name_suffix": "_r125", "priority": -31},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 45899,
            "min_samples_leaf": 20,
            "ag_args": {"name_suffix": "_r50", "priority": -46},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 18704,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r144", "priority": -57},
        },
        {
            "max_features": 0.75,
            "max_leaf_nodes": 48569,
            "min_samples_leaf": 40,
            "ag_args": {"name_suffix": "_r200", "priority": -66},
        },
        {
            "max_features": "sqrt",
            "max_leaf_nodes": 11995,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r148", "priority": -74},
        },
        {
            "max_features": 0.75,
            "max_leaf_nodes": 36230,
            "min_samples_leaf": 3,
            "ag_args": {"name_suffix": "_r15", "priority": -79},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 26368,
            "min_samples_leaf": 2,
            "ag_args": {"name_suffix": "_r19", "priority": -106},
        },
        {
            "max_features": 0.5,
            "max_leaf_nodes": 22329,
            "min_samples_leaf": 20,
            "ag_args": {"name_suffix": "_r111", "priority": -126},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 42420,
            "min_samples_leaf": 10,
            "ag_args": {"name_suffix": "_r83", "priority": -140},
        },
        {
            "max_features": 0.5,
            "max_leaf_nodes": 43008,
            "min_samples_leaf": 5,
            "ag_args": {"name_suffix": "_r95", "priority": -141},
        },
        {
            "max_features": 0.5,
            "max_leaf_nodes": 19705,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r134", "priority": -150},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 34081,
            "min_samples_leaf": 3,
            "ag_args": {"name_suffix": "_r194", "priority": -178},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 12845,
            "min_samples_leaf": 4,
            "ag_args": {"name_suffix": "_r172", "priority": -179},
        },
        {
            "max_features": "log2",
            "max_leaf_nodes": 42644,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r166", "priority": -180},
        },
        {
            "max_features": "sqrt",
            "max_leaf_nodes": 30080,
            "min_samples_leaf": 80,
            "ag_args": {"name_suffix": "_r113", "priority": -187},
        },
    ],
    "XT": [
        {
            "criterion": "gini",
            "ag_args": {
                "name_suffix": "Gini",
                "problem_types": ["binary", "multiclass"],
            },
        },
        {
            "criterion": "entropy",
            "ag_args": {
                "name_suffix": "Entr",
                "problem_types": ["binary", "multiclass"],
            },
        },
        {
            "criterion": "squared_error",
            "ag_args": {
                "name_suffix": "MSE",
                "problem_types": ["regression", "quantile"],
            },
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 18729,
            "min_samples_leaf": 5,
            "ag_args": {"name_suffix": "_r137", "priority": -16},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 15825,
            "min_samples_leaf": 3,
            "ag_args": {"name_suffix": "_r196", "priority": -61},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 48136,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r16", "priority": -72},
        },
        {
            "max_features": 0.75,
            "max_leaf_nodes": 46988,
            "min_samples_leaf": 40,
            "ag_args": {"name_suffix": "_r102", "priority": -75},
        },
        {
            "max_features": "log2",
            "max_leaf_nodes": 33436,
            "min_samples_leaf": 2,
            "ag_args": {"name_suffix": "_r119", "priority": -86},
        },
        {
            "max_features": 0.75,
            "max_leaf_nodes": 12012,
            "min_samples_leaf": 5,
            "ag_args": {"name_suffix": "_r25", "priority": -89},
        },
        {
            "max_features": 0.5,
            "max_leaf_nodes": 30392,
            "min_samples_leaf": 4,
            "ag_args": {"name_suffix": "_r31", "priority": -99},
        },
        {
            "max_features": 0.75,
            "max_leaf_nodes": 37308,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r195", "priority": -128},
        },
        {
            "max_features": "log2",
            "max_leaf_nodes": 24441,
            "min_samples_leaf": 20,
            "ag_args": {"name_suffix": "_r123", "priority": -155},
        },
        {
            "max_features": 1.0,
            "max_leaf_nodes": 31951,
            "min_samples_leaf": 4,
            "ag_args": {"name_suffix": "_r142", "priority": -169},
        },
        {
            "max_features": 0.5,
            "max_leaf_nodes": 36718,
            "min_samples_leaf": 1,
            "ag_args": {"name_suffix": "_r158", "priority": -177},
        },
    ],
    "KNN": [
        {"weights": "uniform", "ag_args": {"name_suffix": "Unif"}},
        {"weights": "distance", "ag_args": {"name_suffix": "Dist"}},
        {
            "weights": "distance",
            "p": 2,
            "n_neighbors": 50,
            "ag_args": {"name_suffix": "_r32", "priority": -56},
        },
        {
            "weights": "distance",
            "p": 1,
            "n_neighbors": 5,
            "ag_args": {"name_suffix": "_r16", "priority": -82},
        },
        {
            "weights": "uniform",
            "p": 1,
            "n_neighbors": 3,
            "ag_args": {"name_suffix": "_r23", "priority": -136},
        },
        {
            "weights": "uniform",
            "p": 1,
            "n_neighbors": 30,
            "ag_args": {"name_suffix": "_r45", "priority": -151},
        },
    ],
    "LR": [
        {
            "C": 978.6204803985407,
            "penalty": "L2",
            "proc.impute_strategy": "mean",
            "proc.skew_threshold": None,
            "ag_args": {"name_suffix": "_r9", "priority": 95},
        },
        {
            "C": 958.9533736976579,
            "penalty": "L1",
            "proc.impute_strategy": "mean",
            "proc.skew_threshold": 0.99,
            "ag_args": {"name_suffix": "_r22", "priority": 15},
        },
        {
            "C": 55.80912223223615,
            "penalty": "L1",
            "proc.impute_strategy": "mean",
            "proc.skew_threshold": None,
            "ag_args": {"name_suffix": "_r30", "priority": -21},
        },
        {
            "C": 647.2094227735367,
            "penalty": "L2",
            "proc.impute_strategy": "median",
            "proc.skew_threshold": 0.999,
            "ag_args": {"name_suffix": "_r38", "priority": -36},
        },
        {
            "C": 857.9598230609945,
            "penalty": "L1",
            "proc.impute_strategy": "mean",
            "proc.skew_threshold": None,
            "ag_args": {"name_suffix": "_r2", "priority": -67},
        },
        {
            "C": 791.7458655788563,
            "penalty": "L1",
            "proc.impute_strategy": "median",
            "proc.skew_threshold": 0.99,
            "ag_args": {"name_suffix": "_r5", "priority": -159},
        },
    ],
}
